/*
 * PWM.h
 *
 *  Created on: 14 janv. 2019
 *      Author: e_rcluze
 */

#ifndef PWM_H_
#define PWM_H_

#include "tim.h"

#define S1 0X01
#define S2 0X02
#define S3 0X03
#define S4 0X04
#define S5 0X05
#define S6 0X06
#define S7 0X07
#define S8 0X08
#define S9 0X09
#define S10 0X0A

#define PWM_MAX 31250-1

void PWMSetDutyCycle(uint32_t sortie, uint16_t value);

void PWMSetZero(uint32_t sortie);

void PWMSetMAX(uint32_t sortie);

void PWMStart(uint8_t sortie);

/**
 * Fonction qui renvoie la valeurs que doit prendre le PWM en fonction de la minutes
 * ex : a 30 min on renvoie la moitier du PWM_MAX.
 */
uint16_t PWMvalue(uint8_t heure);

#endif /* PWM_H_ */
